@@ -27,6 +27,7 @@ from sdaps import defs
 from sdaps.utils.exceptions import RecognitionError
 from sdaps import log
 
+from sdaps.utils.barcode import read_barcode
 from sdaps.utils.ugettext import ugettext, ungettext
 _ = ugettext
 
@@ -35,9 +36,8 @@ pt_to_mm = 25.4 / 72.0
 warned_multipage_not_correctly_scanned = False
 
 
-class Sheet(model.buddy.Buddy):
+class Sheet(model.buddy.Buddy, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.sheet.Sheet
 
@@ -282,7 +282,7 @@ class Sheet(model.buddy.Buddy):
             image.recognize.clean()
 
     def duplex_copy_image_attr(self, failed_pages, attr, error_msg=None):
-        u"""If in duplex mode, this function will copy the given attribute
+        """If in duplex mode, this function will copy the given attribute
         from the image that defines it over to the one that does not.
         ie. if the attribute is None in one and differently in the other image
         it is copied.
@@ -307,9 +307,8 @@ class Sheet(model.buddy.Buddy):
             i += 2
 
 
-class Image(model.buddy.Buddy):
+class Image(model.buddy.Buddy, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.sheet.Image
 
@@ -317,11 +316,11 @@ class Image(model.buddy.Buddy):
         model.buddy.Buddy.__init__(self, *args)
 
         if self.obj.sheet.survey.defs.style == "classic":
-            import classic
+            from . import classic
         elif self.obj.sheet.survey.defs.style == "code128":
-            import code128
+            from . import code128
         elif self.obj.sheet.survey.defs.style == "qr":
-            import qrcode
+            from . import qrcode
         elif self.obj.sheet.survey.defs.style == "custom":
             if not hasattr(self.obj, "style"):
                 import sys
@@ -458,9 +457,8 @@ class Image(model.buddy.Buddy):
     def matrix(self):
         return self.obj.matrix.mm_to_px(fallback=False)
 
-class Questionnaire(model.buddy.Buddy):
+class Questionnaire(model.buddy.Buddy, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.questionnaire.Questionnaire
 
@@ -497,15 +495,15 @@ class Questionnaire(model.buddy.Buddy):
         self.obj.sheet.recognized = True
         # Any newly recognized sheet is definately not verified.
         # This is relevant for reruns.
-        self.obj.sheet.verified = False
+        for img in self.obj.sheet.images:
+            img.verified = False
 
         # clean up
         self.obj.sheet.recognize.clean()
 
 
-class QObject(model.buddy.Buddy):
+class QObject(model.buddy.Buddy, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.questionnaire.QObject
 
@@ -516,9 +514,8 @@ class QObject(model.buddy.Buddy):
         return 1
 
 
-class Question(model.buddy.Buddy):
+class Question(model.buddy.Buddy, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.questionnaire.Question
 
@@ -533,23 +530,9 @@ class Question(model.buddy.Buddy):
             result = min(result, box.data.quality)
         return result
 
-#class Choice(Question):
 
-    #__metaclass__ = model.buddy.Register
-    #name = 'recognize'
-    #obj_class = model.questionnaire.Choice
+class Box(model.buddy.Buddy, metaclass=model.buddy.Register):
 
-
-#class Mark(Question):
-
-    #__metaclass__ = model.buddy.Register
-    #name = 'recognize'
-    #obj_class = model.questionnaire.Mark
-
-
-class Box(model.buddy.Buddy):
-
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.questionnaire.Box
 
@@ -557,15 +540,15 @@ class Box(model.buddy.Buddy):
         pass
 
 
-class Checkbox(Box):
+class Checkbox(Box, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.questionnaire.Checkbox
 
     def prepare_mask(self):
         img = self.obj.sheet.get_page_image(self.obj.page_number)
         width, height = self.obj.width, self.obj.height
+        line_width = self.obj.lw
 
         matrix = list(img.recognize.matrix)
         # Remove any offset from the matrix
@@ -588,7 +571,6 @@ class Checkbox(Box):
 
         cr.set_source_rgba(0, 0, 0, 1)
 
-        line_width = 1 / 72.0 * 25.4
         cr.set_line_width(line_width)
 
         matrix.invert()
@@ -620,6 +602,7 @@ class Checkbox(Box):
         return surf, xoff, yoff
 
     def get_inner_mask(self):
+        """Note this discards half a line width inside the box!"""
         cr, surf, line_width, width, height, xoff, yoff = self.prepare_mask()
 
         if self.obj.form == "ellipse":
@@ -678,7 +661,8 @@ class Checkbox(Box):
         x, y = img.recognize.matrix.transform_point(x, y)
         x, y = int(x), int(y)
 
-        remove_line_width = 1.2 * 25.4 / 72.0
+        # This is not the outline, but the width of the drawn stroke!
+        remove_line_width = 1.2 * pt_to_mm
         remove_line_width_px = max(img.recognize.matrix.transform_distance(remove_line_width, remove_line_width))
 
         coverage = img.recognize.get_masked_coverage(mask, x, y)
@@ -697,7 +681,7 @@ class Checkbox(Box):
         state = 0
         quality = -1
         # Iterate the ranges
-        for metric, value in self.obj.data.metrics.iteritems():
+        for metric, value in self.obj.data.metrics.items():
             metric = defs.checkbox_metrics[self.obj.sheet.survey.defs.checkmode][metric]
 
             for lower, upper in zip(metric[:-1], metric[1:]):
@@ -716,9 +700,8 @@ class Checkbox(Box):
         self.obj.data.quality = min(quality, pos_quality)
 
 
-class Textbox(Box):
+class Textbox(Box, metaclass=model.buddy.Register):
 
-    __metaclass__ = model.buddy.Register
     name = 'recognize'
     obj_class = model.questionnaire.Textbox
 
@@ -772,7 +755,7 @@ class Textbox(Box):
                 dist_y = dest_y - y
 
                 length = math.sqrt(dist_x ** 2 + dist_y ** 2)
-                for step in xrange(int(length / step_x)):
+                for step in range(int(length / step_x)):
                     yield x + dist_x * step / (length / step_x), y + dist_y * step / (length / step_x)
                 yield dest_x, dest_y
 
@@ -788,7 +771,7 @@ class Textbox(Box):
                 dist_y = dest_y - y
 
                 length = math.sqrt(dist_x ** 2 + dist_y ** 2)
-                for step in xrange(int(length / step_x)):
+                for step in range(int(length / step_x)):
                     yield x + dist_x * step / (length / step_x), y + dist_y * step / (length / step_x)
                 yield dest_x, dest_y
 
@@ -804,7 +787,7 @@ class Textbox(Box):
                 dist_y = dest_y - y
 
                 length = math.sqrt(dist_x ** 2 + dist_y ** 2)
-                for step in xrange(int(length / step_y)):
+                for step in range(int(length / step_y)):
                     yield x + dist_x * step / (length / step_y), y + dist_y * step / (length / step_y)
                 yield dest_x, dest_y
 
@@ -820,7 +803,7 @@ class Textbox(Box):
                 dist_y = dest_y - y
 
                 length = math.sqrt(dist_x ** 2 + dist_y ** 2)
-                for step in xrange(int(length / step_y)):
+                for step in range(int(length / step_y)):
                     yield x + dist_x * step / (length / step_y), y + dist_y * step / (length / step_y)
                 yield dest_x, dest_y
 
@@ -909,3 +892,36 @@ class Textbox(Box):
             self.obj.data.y = self.obj.y
             self.obj.data.width = self.obj.width
             self.obj.data.height = self.obj.height
+
+class Codebox(Textbox, metaclass=model.buddy.Register):
+
+    name = 'recognize'
+    obj_class = model.questionnaire.Codebox
+
+    def recognize(self):
+        img = self.obj.sheet.get_page_image(self.obj.page_number)
+
+        if img is None or img.recognize.matrix is None:
+            self.obj.sheet.valid = 0
+            return
+
+        x = self.obj.x
+        y = self.obj.y
+        width = self.obj.width
+        height = self.obj.height
+
+        surface = img.surface.surface
+        matrix = img.recognize.matrix
+
+        res = read_barcode(surface, matrix,
+                           x, y, width, height,
+                           "QRCODE")
+
+        # This is maybe a bit odd, but we accept empty string as a valid
+        # content (if there was a barcode which was just the empty string).
+        if res is not None:
+            self.obj.data.state = True
+            self.obj.data.text = res
+        else:
+            self.obj.data.state = False
+            self.obj.data.text = None
